@@ -2,7 +2,7 @@ static char module_doc[] =
 "This module provides a BLAS optimized\nmatrix multiply, inner product and dot for numpy arrays";
 
 #include "Python.h"
-#include "numpy/noprefix.h"
+#include "numpy/ndarrayobject.h"
 #ifndef CBLAS_HEADER
 #define CBLAS_HEADER "cblas.h"
 #endif
@@ -13,14 +13,14 @@ static char module_doc[] =
 static PyArray_DotFunc *oldFunctions[PyArray_NTYPES];
 
 static void
-FLOAT_dot(void *a, intp stridea, void *b, intp strideb, void *res,
-	  intp n, void *tmp)
+FLOAT_dot(void *a, npy_intp stridea, void *b, npy_intp strideb, void *res,
+	  npy_intp n, void *tmp)
 {
-    register int na = stridea / sizeof(float);
-    register int nb = strideb / sizeof(float);
+    register npy_intp na = stridea / sizeof(float);
+    register npy_intp nb = strideb / sizeof(float);
 
-    if ((sizeof(float) * na == stridea) &&
-	(sizeof(float) * nb == strideb) &&
+    if ((sizeof(float) * na == (size_t)stridea) &&
+	(sizeof(float) * nb == (size_t)strideb) &&
 	(na >= 0) && (nb >= 0))
 	    *((float *)res) = cblas_sdot((int)n, (float *)a, na, (float *)b, nb);
 
@@ -29,14 +29,14 @@ FLOAT_dot(void *a, intp stridea, void *b, intp strideb, void *res,
 }
 
 static void
-DOUBLE_dot(void *a, intp stridea, void *b, intp strideb, void *res,
-	   intp n, void *tmp)
+DOUBLE_dot(void *a, npy_intp stridea, void *b, npy_intp strideb, void *res,
+	   npy_intp n, void *tmp)
 {
     register int na = stridea / sizeof(double);
     register int nb = strideb / sizeof(double);
 
-    if ((sizeof(double) * na == stridea) &&
-	(sizeof(double) * nb == strideb) &&
+    if ((sizeof(double) * na == (size_t)stridea) &&
+	(sizeof(double) * nb == (size_t)strideb) &&
 	(na >= 0) && (nb >= 0))
 	    *((double *)res) = cblas_ddot((int)n, (double *)a, na, (double *)b, nb);
     else
@@ -44,15 +44,15 @@ DOUBLE_dot(void *a, intp stridea, void *b, intp strideb, void *res,
 }
 
 static void
-CFLOAT_dot(void *a, intp stridea, void *b, intp strideb, void *res,
-	   intp n, void *tmp)
+CFLOAT_dot(void *a, npy_intp stridea, void *b, npy_intp strideb, void *res,
+	   npy_intp n, void *tmp)
 {
 
-    register int na = stridea / sizeof(cfloat);
-    register int nb = strideb / sizeof(cfloat);
+    register int na = stridea / sizeof(npy_cfloat);
+    register int nb = strideb / sizeof(npy_cfloat);
 
-    if ((sizeof(cfloat) * na == stridea) &&
-	(sizeof(cfloat) * nb == strideb) &&
+    if ((sizeof(npy_cfloat) * na == (size_t)stridea) &&
+	(sizeof(npy_cfloat) * nb == (size_t)strideb) &&
 	(na >= 0) && (nb >= 0))
 	    cblas_cdotu_sub((int)n, (float *)a, na, (float *)b, nb, (float *)res);
     else
@@ -60,14 +60,14 @@ CFLOAT_dot(void *a, intp stridea, void *b, intp strideb, void *res,
 }
 
 static void
-CDOUBLE_dot(void *a, intp stridea, void *b, intp strideb, void *res,
-	    intp n, void *tmp)
+CDOUBLE_dot(void *a, npy_intp stridea, void *b, npy_intp strideb, void *res,
+	    npy_intp n, void *tmp)
 {
-    register int na = stridea / sizeof(cdouble);
-    register int nb = strideb / sizeof(cdouble);
+    register int na = stridea / sizeof(npy_cdouble);
+    register int nb = strideb / sizeof(npy_cdouble);
 
-    if ((sizeof(cdouble) * na == stridea) &&
-	(sizeof(cdouble) * nb == strideb) &&
+    if ((sizeof(npy_cdouble) * na == (size_t)stridea) &&
+	(sizeof(npy_cdouble) * nb == (size_t)strideb) &&
 	(na >= 0) && (nb >= 0))
 	    cblas_zdotu_sub((int)n, (double *)a, na, (double *)b, nb, (double *)res);
     else
@@ -75,12 +75,13 @@ CDOUBLE_dot(void *a, intp stridea, void *b, intp strideb, void *res,
 }
 
 
-static Bool altered=FALSE;
-
-static char doc_alterdot[] = "alterdot() changes all dot functions to use blas.";
+static npy_bool altered=NPY_FALSE;
 
+/*
+ * alterdot() changes all dot functions to use blas.
+ */
 static PyObject *
-dotblas_alterdot(PyObject *dummy, PyObject *args)
+dotblas_alterdot(PyObject *NPY_UNUSED(dummy), PyObject *args)
 {
     PyArray_Descr *descr;
 
@@ -105,17 +106,18 @@ dotblas_alterdot(PyObject *dummy, PyObject *args)
 	oldFunctions[PyArray_CDOUBLE] = descr->f->dotfunc;
 	descr->f->dotfunc = (PyArray_DotFunc *)CDOUBLE_dot;
 
-	altered = TRUE;
+	altered = NPY_TRUE;
     }
 
     Py_INCREF(Py_None);
     return Py_None;
 }
 
-static char doc_restoredot[] = "restoredot() restores dots to defaults.";
-
+/*
+ * restoredot() restores dots to defaults.
+ */
 static PyObject *
-dotblas_restoredot(PyObject *dummy, PyObject *args)
+dotblas_restoredot(PyObject *NPY_UNUSED(dummy), PyObject *args)
 {
     PyArray_Descr *descr;
 
@@ -142,7 +144,7 @@ dotblas_restoredot(PyObject *dummy, PyObject *args)
 	oldFunctions[PyArray_CDOUBLE] = NULL;
 	Py_XDECREF(descr);
 
-	altered = FALSE;
+	altered = NPY_FALSE;
     }
 
     Py_INCREF(Py_None);
@@ -184,9 +186,9 @@ _bad_strides(PyArrayObject *ap)
 {
     register int itemsize = PyArray_ITEMSIZE(ap);
     register int i, N=PyArray_NDIM(ap);
-    register intp *strides = PyArray_STRIDES(ap);
+    register npy_intp *strides = PyArray_STRIDES(ap);
 
-    if (((intp)(ap->data) % itemsize) != 0)
+    if (((npy_intp)(ap->data) % itemsize) != 0)
 	return 1;
     for (i=0; i<N; i++) {
 	if ((strides[i] < 0) || (strides[i] % itemsize) != 0)
@@ -196,22 +198,23 @@ _bad_strides(PyArrayObject *ap)
     return 0;
 }
 
-static char doc_matrixproduct[] = \
-    "dot(a,b)\nReturns the dot product of a and b for arrays of " \
-    "floating point types.\nLike the generic numpy equivalent the " \
-    "product sum is over\nthe last dimension of a and the second-to-last "\
-    "dimension of b.\nNB: The first argument is not conjugated.";
-
+/*
+ * dot(a,b)
+ * Returns the dot product of a and b for arrays of floating point types.
+ * Like the generic numpy equivalent the product sum is over
+ * the last dimension of a and the second-to-last dimension of b.
+ * NB: The first argument is not conjugated.;
+ */
 static PyObject *
-dotblas_matrixproduct(PyObject *dummy, PyObject *args)
+dotblas_matrixproduct(PyObject *NPY_UNUSED(dummy), PyObject *args)
 {
     PyObject *op1, *op2;
     PyArrayObject *ap1 = NULL, *ap2 = NULL, *ret = NULL;
     int j, l, lda, ldb, ldc;
     int typenum, nd;
-    intp ap1stride = 0;
-    intp dimensions[MAX_DIMS];
-    intp numbytes;
+    npy_intp ap1stride = 0;
+    npy_intp dimensions[NPY_MAXDIMS];
+    npy_intp numbytes;
     static const float oneF[2] = {1.0, 0.0};
     static const float zeroF[2] = {0.0, 0.0};
     static const double oneD[2] = {1.0, 0.0};
@@ -244,12 +247,12 @@ dotblas_matrixproduct(PyObject *dummy, PyObject *args)
         return NULL;
     }
     Py_INCREF(dtype);
-    ap1 = (PyArrayObject *)PyArray_FromAny(op1, dtype, 0, 0, ALIGNED, NULL);
+    ap1 = (PyArrayObject *)PyArray_FromAny(op1, dtype, 0, 0, NPY_ALIGNED, NULL);
     if (ap1 == NULL) {
         Py_DECREF(dtype);
         return NULL;
     }
-    ap2 = (PyArrayObject *)PyArray_FromAny(op2, dtype, 0, 0, ALIGNED, NULL);
+    ap2 = (PyArrayObject *)PyArray_FromAny(op2, dtype, 0, 0, NPY_ALIGNED, NULL);
     if (ap2 == NULL) {
         Py_DECREF(ap1);
         return NULL;
@@ -315,7 +318,7 @@ dotblas_matrixproduct(PyObject *dummy, PyObject *args)
         }
 
  	if (ap1->nd == 0 || ap2->nd == 0) {
-            intp *thisdims;
+            npy_intp *thisdims;
             if (ap1->nd == 0) {
                 nd = ap2->nd;
                 thisdims = ap2->dimensions;
@@ -369,6 +372,11 @@ dotblas_matrixproduct(PyObject *dummy, PyObject *args)
                     l = dimensions[0];
                 }
             }
+
+            /* Check if the summation dimension is 0-sized */
+            if (oap1->dimensions[oap1->nd - 1] == 0) {
+                l = 0;
+            }
 	}
     }
     else {
@@ -460,17 +468,17 @@ dotblas_matrixproduct(PyObject *dummy, PyObject *args)
 	}
 	else if (typenum == PyArray_CDOUBLE) {
 	    if (l == 1) {
-		cdouble *ptr1, *ptr2, *res;
+		npy_cdouble *ptr1, *ptr2, *res;
 
-		ptr1 = (cdouble *)ap2->data;
-		ptr2 = (cdouble *)ap1->data;
-		res = (cdouble *)ret->data;
+		ptr1 = (npy_cdouble *)ap2->data;
+		ptr2 = (npy_cdouble *)ap1->data;
+		res = (npy_cdouble *)ret->data;
 		res->real = ptr1->real * ptr2->real - ptr1->imag * ptr2->imag;
 		res->imag = ptr1->real * ptr2->imag + ptr1->imag * ptr2->real;
 	    }
 	    else if (ap1shape != _matrix) {
 		cblas_zaxpy(l, (double *)ap2->data, (double *)ap1->data,
-			    ap1stride/sizeof(cdouble), (double *)ret->data, 1);
+			    ap1stride/sizeof(npy_cdouble), (double *)ret->data, 1);
 	    }
 	    else {
 		int maxind, oind, i, a1s, rets;
@@ -483,8 +491,8 @@ dotblas_matrixproduct(PyObject *dummy, PyObject *args)
 		rptr = ret->data;
 		l = ap1->dimensions[maxind];
 		pval = (double *)ap2->data;
-		a1s = ap1->strides[maxind] / sizeof(cdouble);
-		rets = ret->strides[maxind] / sizeof(cdouble);
+		a1s = ap1->strides[maxind] / sizeof(npy_cdouble);
+		rets = ret->strides[maxind] / sizeof(npy_cdouble);
 		for (i = 0; i < ap1->dimensions[oind]; i++) {
 		    cblas_zaxpy(l, pval, (double *)ptr, a1s,
 				(double *)rptr, rets);
@@ -525,17 +533,17 @@ dotblas_matrixproduct(PyObject *dummy, PyObject *args)
 	}
 	else if (typenum == PyArray_CFLOAT) {
 	    if (l == 1) {
-		cfloat *ptr1, *ptr2, *res;
+		npy_cfloat *ptr1, *ptr2, *res;
 
-		ptr1 = (cfloat *)ap2->data;
-		ptr2 = (cfloat *)ap1->data;
-		res = (cfloat *)ret->data;
+		ptr1 = (npy_cfloat *)ap2->data;
+		ptr2 = (npy_cfloat *)ap1->data;
+		res = (npy_cfloat *)ret->data;
 		res->real = ptr1->real * ptr2->real - ptr1->imag * ptr2->imag;
 		res->imag = ptr1->real * ptr2->imag + ptr1->imag * ptr2->real;
 	    }
 	    else if (ap1shape != _matrix) {
 		cblas_caxpy(l, (float *)ap2->data, (float *)ap1->data,
-			    ap1stride/sizeof(cfloat), (float *)ret->data, 1);
+			    ap1stride/sizeof(npy_cfloat), (float *)ret->data, 1);
 	    }
 	    else {
 		int maxind, oind, i, a1s, rets;
@@ -548,8 +556,8 @@ dotblas_matrixproduct(PyObject *dummy, PyObject *args)
 		rptr = ret->data;
 		l = ap1->dimensions[maxind];
 		pval = (float *)ap2->data;
-		a1s = ap1->strides[maxind] / sizeof(cfloat);
-		rets = ret->strides[maxind] / sizeof(cfloat);
+		a1s = ap1->strides[maxind] / sizeof(npy_cfloat);
+		rets = ret->strides[maxind] / sizeof(npy_cfloat);
 		for (i = 0; i < ap1->dimensions[oind]; i++) {
 		    cblas_caxpy(l, pval, (float *)ptr, a1s,
 				(float *)rptr, rets);
@@ -791,20 +799,23 @@ dotblas_matrixproduct(PyObject *dummy, PyObject *args)
 }
 
 
-static char doc_innerproduct[] = \
-    "innerproduct(a,b)\nReturns the inner product of a and b for arrays of "\
-    "floating point types.\nLike the generic NumPy equivalent the product "\
-    "sum is over\nthe last dimension of a and b.\nNB: The first argument is "\
-    "not conjugated.";
+/*
+ * innerproduct(a,b)
+ *
+ * Returns the inner product of a and b for arrays of
+ * floating point types. Like the generic NumPy equivalent the product
+ * sum is over the last dimension of a and b.
+ * NB: The first argument is not conjugated.
+ */
 
 static PyObject *
-dotblas_innerproduct(PyObject *dummy, PyObject *args)
+dotblas_innerproduct(PyObject *NPY_UNUSED(dummy), PyObject *args)
 {
     PyObject *op1, *op2;
     PyArrayObject *ap1, *ap2, *ret;
     int j, l, lda, ldb, ldc;
     int typenum, nd;
-    intp dimensions[MAX_DIMS];
+    npy_intp dimensions[NPY_MAXDIMS];
     static const float oneF[2] = {1.0, 0.0};
     static const float zeroF[2] = {0.0, 0.0};
     static const double oneD[2] = {1.0, 0.0};
@@ -1043,15 +1054,18 @@ dotblas_innerproduct(PyObject *dummy, PyObject *args)
 }
 
 
-static char doc_vdot[] = "vdot(a,b)\nReturns the dot product of a and b for scalars and vectors\nof floating point and complex types.  The first argument, a, is conjugated.";
-
-
-static PyObject *dotblas_vdot(PyObject *dummy, PyObject *args) {
+/*
+ * vdot(a,b)
+ *
+ * Returns the dot product of a and b for scalars and vectors of
+ * floating point and complex types.  The first argument, a, is conjugated.
+ */
+static PyObject *dotblas_vdot(PyObject *NPY_UNUSED(dummy), PyObject *args) {
     PyObject *op1, *op2;
     PyArrayObject *ap1=NULL, *ap2=NULL, *ret=NULL;
     int l;
     int typenum;
-    intp dimensions[MAX_DIMS];
+    npy_intp dimensions[NPY_MAXDIMS];
     PyArray_Descr *type;
 
     if (!PyArg_ParseTuple(args, "OO", &op1, &op2)) return NULL;
@@ -1146,11 +1160,11 @@ static PyObject *dotblas_vdot(PyObject *dummy, PyObject *args) {
 }
 
 static struct PyMethodDef dotblas_module_methods[] = {
-    {"dot",  (PyCFunction)dotblas_matrixproduct, 1, doc_matrixproduct},
-    {"inner",   (PyCFunction)dotblas_innerproduct,  1, doc_innerproduct},
-    {"vdot", (PyCFunction)dotblas_vdot, 1, doc_vdot},
-    {"alterdot", (PyCFunction)dotblas_alterdot, 1, doc_alterdot},
-    {"restoredot", (PyCFunction)dotblas_restoredot, 1, doc_restoredot},
+    {"dot",  (PyCFunction)dotblas_matrixproduct, 1, NULL},
+    {"inner",   (PyCFunction)dotblas_innerproduct,  1, NULL},
+    {"vdot", (PyCFunction)dotblas_vdot, 1, NULL},
+    {"alterdot", (PyCFunction)dotblas_alterdot, 1, NULL},
+    {"restoredot", (PyCFunction)dotblas_restoredot, 1, NULL},
     {NULL, NULL, 0, NULL}		/* sentinel */
 };
 
