@@ -1,10 +1,24 @@
-import re
+import re, textwrap
+import sys, os
+sys.path.insert(0, os.path.dirname(__file__))
+import ufunc_docstrings as docstrings
+sys.path.pop(0)
 
 Zero = "PyUFunc_Zero"
 One = "PyUFunc_One"
 None_ = "PyUFunc_None"
 
 class TypeDescription(object):
+    """Type signature for a ufunc
+
+    Attributes
+    ----------
+
+    type: character representing the type
+    func_data:
+    in_:
+    out:
+    """
     def __init__(self, type, f=None, in_=None, out=None):
         self.type = type
         self.func_data = f
@@ -23,7 +37,7 @@ class TypeDescription(object):
             self.out = self.type * nout
         assert len(self.out) == nout
 
-_fdata_map = dict(f='%sf', d='%s', g='%sl',
+_fdata_map = dict(f='npy_%sf', d='npy_%s', g='npy_%sl',
                   F='nc_%sf', D='nc_%s', G='nc_%sl')
 def build_func_data(types, f):
     func_data = []
@@ -55,6 +69,17 @@ def TD(types, f=None, in_=None, out=None):
     return tds
 
 class Ufunc(object):
+    """Description of a ufunc.
+
+    Attributes
+    ----------
+
+    nin: number of input arguments
+    nout: number of output arguments
+    identity: identity element for a two-argument function
+    docstring: docstring for the ufunc
+    type_descriptions: list of TypeDescription objects
+    """
     def __init__(self, nin, nout, identity, docstring,
                  *type_descriptions):
         self.nin = nin
@@ -69,7 +94,7 @@ class Ufunc(object):
         for td in self.type_descriptions:
             td.finish_signature(self.nin, self.nout)
 
-#each entry in defdict is
+#each entry in defdict is a Ufunc object.
 
 #name: [string of chars for which it is defined,
 #       string of characters using func interface,
@@ -80,6 +105,29 @@ class Ufunc(object):
 #       output specification (optional)
 #       ]
 
+chartoname = {'?': 'bool',
+              'b': 'byte',
+              'B': 'ubyte',
+              'h': 'short',
+              'H': 'ushort',
+              'i': 'int',
+              'I': 'uint',
+              'l': 'long',
+              'L': 'ulong',
+              'q': 'longlong',
+              'Q': 'ulonglong',
+              'f': 'float',
+              'd': 'double',
+              'g': 'longdouble',
+              'F': 'cfloat',
+              'D': 'cdouble',
+              'G': 'clongdouble',
+              'O': 'OBJECT',
+              # M is like O, but calls a method of the object instead
+              # of a function
+              'M': 'OBJECT',
+              }
+
 all = '?bBhHiIlLqQfdgFDGO'
 O = 'O'
 M = 'M'
@@ -109,37 +157,37 @@ noobj = all[:-1]
 defdict = {
 'add' :
     Ufunc(2, 1, Zero,
-          'adds the arguments elementwise.',
+          docstrings.get('numpy.core.umath.add'),
           TD(noobj),
           TD(O, f='PyNumber_Add'),
           ),
 'subtract' :
     Ufunc(2, 1, Zero,
-          'subtracts the arguments elementwise.',
+          docstrings.get('numpy.core.umath.subtract'),
           TD(noobj),
           TD(O, f='PyNumber_Subtract'),
           ),
 'multiply' :
     Ufunc(2, 1, One,
-          'multiplies the arguments elementwise.',
+          docstrings.get('numpy.core.umath.multiply'),
           TD(noobj),
           TD(O, f='PyNumber_Multiply'),
           ),
 'divide' :
     Ufunc(2, 1, One,
-          'divides the arguments elementwise.',
+          docstrings.get('numpy.core.umath.divide'),
           TD(intfltcmplx),
           TD(O, f='PyNumber_Divide'),
           ),
 'floor_divide' :
     Ufunc(2, 1, One,
-          'floor divides the arguments elementwise.',
+          docstrings.get('numpy.core.umath.floor_divide'),
           TD(intfltcmplx),
           TD(O, f='PyNumber_FloorDivide'),
           ),
 'true_divide' :
     Ufunc(2, 1, One,
-          'true divides the arguments elementwise.',
+          docstrings.get('numpy.core.umath.true_divide'),
           TD('bBhH', out='f'),
           TD('iIlLqQ', out='d'),
           TD(flts+cmplx),
@@ -147,336 +195,396 @@ defdict = {
           ),
 'conjugate' :
     Ufunc(1, 1, None,
-          'takes the conjugate of x elementwise.',
+          docstrings.get('numpy.core.umath.conjugate'),
           TD(nobool_or_obj),
           TD(M, f='conjugate'),
           ),
 'fmod' :
     Ufunc(2, 1, Zero,
-          'computes (C-like) x1 % x2 elementwise.',
+          docstrings.get('numpy.core.umath.fmod'),
           TD(ints),
           TD(flts, f='fmod'),
           TD(M, f='fmod'),
           ),
 'square' :
     Ufunc(1, 1, None,
-          'compute x**2.',
+          docstrings.get('numpy.core.umath.square'),
           TD(nobool_or_obj),
           TD(O, f='Py_square'),
           ),
 'reciprocal' :
     Ufunc(1, 1, None,
-          'compute 1/x',
+          docstrings.get('numpy.core.umath.reciprocal'),
           TD(nobool_or_obj),
           TD(O, f='Py_reciprocal'),
           ),
 'ones_like' :
     Ufunc(1, 1, None,
-          'returns an array of ones of the shape and typecode of x.',
-          TD(nobool_or_obj),
+          docstrings.get('numpy.core.umath.ones_like'),
+          TD(noobj),
           TD(O, f='Py_get_one'),
           ),
 'power' :
     Ufunc(2, 1, One,
-          'computes x1**x2 elementwise.',
+          docstrings.get('numpy.core.umath.power'),
           TD(ints),
           TD(inexact, f='pow'),
-          TD(O, f='PyNumber_Power'),
+          TD(O, f='npy_ObjectPower'),
           ),
 'absolute' :
     Ufunc(1, 1, None,
-          'takes |x| elementwise.',
+          docstrings.get('numpy.core.umath.absolute'),
           TD(nocmplx),
           TD(cmplx, out=('f', 'd', 'g')),
           TD(O, f='PyNumber_Absolute'),
           ),
 'negative' :
     Ufunc(1, 1, None,
-          'determines -x elementwise',
+          docstrings.get('numpy.core.umath.negative'),
           TD(nocmplx),
           TD(cmplx, f='neg'),
           TD(O, f='PyNumber_Negative'),
           ),
 'sign' :
     Ufunc(1, 1, None,
-          'returns -1 if x < 0 and 0 if x==0 and 1 if x > 0',
+          docstrings.get('numpy.core.umath.sign'),
           TD(nobool),
           ),
 'greater' :
     Ufunc(2, 1, None,
-          'returns elementwise x1 > x2 in a bool array.',
+          docstrings.get('numpy.core.umath.greater'),
           TD(all, out='?'),
           ),
 'greater_equal' :
     Ufunc(2, 1, None,
-          'returns elementwise x1 >= x2 in a bool array.',
+          docstrings.get('numpy.core.umath.greater_equal'),
           TD(all, out='?'),
           ),
 'less' :
     Ufunc(2, 1, None,
-          'returns elementwise x1 < x2 in a bool array.',
+          docstrings.get('numpy.core.umath.less'),
           TD(all, out='?'),
           ),
 'less_equal' :
     Ufunc(2, 1, None,
-          'returns elementwise x1 <= x2 in a bool array',
+          docstrings.get('numpy.core.umath.less_equal'),
           TD(all, out='?'),
           ),
 'equal' :
     Ufunc(2, 1, None,
-          'returns elementwise x1 == x2 in a bool array',
+          docstrings.get('numpy.core.umath.equal'),
           TD(all, out='?'),
           ),
 'not_equal' :
     Ufunc(2, 1, None,
-          'returns elementwise x1 |= x2',
+          docstrings.get('numpy.core.umath.not_equal'),
           TD(all, out='?'),
           ),
 'logical_and' :
     Ufunc(2, 1, One,
-          'returns x1 and x2 elementwise.',
+          docstrings.get('numpy.core.umath.logical_and'),
           TD(noobj, out='?'),
-          TD(M, f='logical_and', out='?'),
+          TD(M, f='logical_and'),
           ),
 'logical_not' :
     Ufunc(1, 1, None,
-          'returns not x elementwise.',
+          docstrings.get('numpy.core.umath.logical_not'),
           TD(noobj, out='?'),
-          TD(M, f='logical_not', out='?'),
+          TD(M, f='logical_not'),
           ),
 'logical_or' :
     Ufunc(2, 1, Zero,
-          'returns x1 or x2 elementwise.',
+          docstrings.get('numpy.core.umath.logical_or'),
           TD(noobj, out='?'),
-          TD(M, f='logical_or', out='?'),
+          TD(M, f='logical_or'),
           ),
 'logical_xor' :
     Ufunc(2, 1, None,
-          'returns x1 xor x2 elementwise.',
+          docstrings.get('numpy.core.umath.logical_xor'),
           TD(noobj, out='?'),
-          TD(M, f='logical_xor', out='?'),
+          TD(M, f='logical_xor'),
           ),
 'maximum' :
     Ufunc(2, 1, None,
-          'returns maximum (if x1 > x2: x1;  else: x2) elementwise.',
+          docstrings.get('numpy.core.umath.maximum'),
           TD(noobj),
-          TD(O, f='_npy_ObjectMax')
+          TD(O, f='npy_ObjectMax')
           ),
 'minimum' :
     Ufunc(2, 1, None,
-          'returns minimum (if x1 < x2: x1;  else: x2) elementwise',
+          docstrings.get('numpy.core.umath.minimum'),
           TD(noobj),
-          TD(O, f='_npy_ObjectMin')
+          TD(O, f='npy_ObjectMin')
+          ),
+'fmax' :
+    Ufunc(2, 1, None,
+          docstrings.get('numpy.core.umath.fmax'),
+          TD(noobj),
+          TD(O, f='npy_ObjectMax')
+          ),
+'fmin' :
+    Ufunc(2, 1, None,
+          docstrings.get('numpy.core.umath.fmin'),
+          TD(noobj),
+          TD(O, f='npy_ObjectMin')
+          ),
+'logaddexp' :
+    Ufunc(2, 1, None,
+          docstrings.get('numpy.core.umath.logaddexp'),
+          TD(flts, f="logaddexp")
+          ),
+'logaddexp2' :
+    Ufunc(2, 1, None,
+          docstrings.get('numpy.core.umath.logaddexp2'),
+          TD(flts, f="logaddexp2")
           ),
 'bitwise_and' :
     Ufunc(2, 1, One,
-          'computes x1 & x2 elementwise.',
+          docstrings.get('numpy.core.umath.bitwise_and'),
           TD(bints),
           TD(O, f='PyNumber_And'),
           ),
 'bitwise_or' :
     Ufunc(2, 1, Zero,
-          'computes x1 | x2 elementwise.',
+          docstrings.get('numpy.core.umath.bitwise_or'),
           TD(bints),
           TD(O, f='PyNumber_Or'),
           ),
 'bitwise_xor' :
     Ufunc(2, 1, None,
-          'computes x1 ^ x2 elementwise.',
+          docstrings.get('numpy.core.umath.bitwise_xor'),
           TD(bints),
           TD(O, f='PyNumber_Xor'),
           ),
 'invert' :
     Ufunc(1, 1, None,
-          'computes ~x (bit inversion) elementwise.',
+          docstrings.get('numpy.core.umath.invert'),
           TD(bints),
           TD(O, f='PyNumber_Invert'),
           ),
 'left_shift' :
     Ufunc(2, 1, None,
-          'computes x1 << x2 (x1 shifted to left by x2 bits) elementwise.',
+          docstrings.get('numpy.core.umath.left_shift'),
           TD(ints),
           TD(O, f='PyNumber_Lshift'),
           ),
 'right_shift' :
     Ufunc(2, 1, None,
-          'computes x1 >> x2 (x1 shifted to right by x2 bits) elementwise.',
+          docstrings.get('numpy.core.umath.right_shift'),
           TD(ints),
           TD(O, f='PyNumber_Rshift'),
           ),
+'degrees' :
+    Ufunc(1, 1, None,
+          docstrings.get('numpy.core.umath.degrees'),
+          TD(fltsM, f='degrees'),
+          ),
+'rad2deg' :
+    Ufunc(1, 1, None,
+          docstrings.get('numpy.core.umath.rad2deg'),
+          TD(fltsM, f='rad2deg'),
+          ),
+'radians' :
+    Ufunc(1, 1, None,
+          docstrings.get('numpy.core.umath.radians'),
+          TD(fltsM, f='radians'),
+          ),
+'deg2rad' :
+    Ufunc(1, 1, None,
+          docstrings.get('numpy.core.umath.deg2rad'),
+          TD(fltsM, f='deg2rad'),
+          ),
 'arccos' :
     Ufunc(1, 1, None,
-          'inverse cosine elementwise.',
+          docstrings.get('numpy.core.umath.arccos'),
           TD(inexact, f='acos'),
           TD(M, f='arccos'),
           ),
 'arccosh' :
     Ufunc(1, 1, None,
-          'inverse hyperbolic cosine elementwise.',
+          docstrings.get('numpy.core.umath.arccosh'),
           TD(inexact, f='acosh'),
           TD(M, f='arccosh'),
           ),
 'arcsin' :
     Ufunc(1, 1, None,
-          'inverse sine elementwise.',
+          docstrings.get('numpy.core.umath.arcsin'),
           TD(inexact, f='asin'),
           TD(M, f='arcsin'),
           ),
 'arcsinh' :
     Ufunc(1, 1, None,
-          'inverse hyperbolic sine elementwise.',
+          docstrings.get('numpy.core.umath.arcsinh'),
           TD(inexact, f='asinh'),
           TD(M, f='arcsinh'),
           ),
 'arctan' :
     Ufunc(1, 1, None,
-          'inverse tangent elementwise.',
+          docstrings.get('numpy.core.umath.arctan'),
           TD(inexact, f='atan'),
           TD(M, f='arctan'),
           ),
 'arctanh' :
     Ufunc(1, 1, None,
-          'inverse hyperbolic tangent elementwise.',
+          docstrings.get('numpy.core.umath.arctanh'),
           TD(inexact, f='atanh'),
           TD(M, f='arctanh'),
           ),
 'cos' :
     Ufunc(1, 1, None,
-          'cosine elementwise.',
+          docstrings.get('numpy.core.umath.cos'),
           TD(inexact, f='cos'),
           TD(M, f='cos'),
           ),
 'sin' :
     Ufunc(1, 1, None,
-          'sine elementwise.',
+          docstrings.get('numpy.core.umath.sin'),
           TD(inexact, f='sin'),
           TD(M, f='sin'),
           ),
 'tan' :
     Ufunc(1, 1, None,
-          'tangent elementwise.',
+          docstrings.get('numpy.core.umath.tan'),
           TD(inexact, f='tan'),
           TD(M, f='tan'),
           ),
 'cosh' :
     Ufunc(1, 1, None,
-          'hyperbolic cosine elementwise.',
+          docstrings.get('numpy.core.umath.cosh'),
           TD(inexact, f='cosh'),
           TD(M, f='cosh'),
           ),
 'sinh' :
     Ufunc(1, 1, None,
-          'hyperbolic sine elementwise.',
+          docstrings.get('numpy.core.umath.sinh'),
           TD(inexact, f='sinh'),
           TD(M, f='sinh'),
           ),
 'tanh' :
     Ufunc(1, 1, None,
-          'hyperbolic tangent elementwise.',
+          docstrings.get('numpy.core.umath.tanh'),
           TD(inexact, f='tanh'),
           TD(M, f='tanh'),
           ),
 'exp' :
     Ufunc(1, 1, None,
-          'e**x elementwise.',
+          docstrings.get('numpy.core.umath.exp'),
           TD(inexact, f='exp'),
           TD(M, f='exp'),
           ),
+'exp2' :
+    Ufunc(1, 1, None,
+          docstrings.get('numpy.core.umath.exp2'),
+          TD(inexact, f='exp2'),
+          TD(M, f='exp2'),
+          ),
 'expm1' :
     Ufunc(1, 1, None,
-          'e**x-1 elementwise.',
+          docstrings.get('numpy.core.umath.expm1'),
           TD(inexact, f='expm1'),
           TD(M, f='expm1'),
           ),
 'log' :
     Ufunc(1, 1, None,
-          'logarithm base e elementwise.',
+          docstrings.get('numpy.core.umath.log'),
           TD(inexact, f='log'),
           TD(M, f='log'),
           ),
+'log2' :
+    Ufunc(1, 1, None,
+          docstrings.get('numpy.core.umath.log2'),
+          TD(inexact, f='log2'),
+          TD(M, f='log2'),
+          ),
 'log10' :
     Ufunc(1, 1, None,
-          'logarithm base 10 elementwise.',
+          docstrings.get('numpy.core.umath.log10'),
           TD(inexact, f='log10'),
           TD(M, f='log10'),
           ),
 'log1p' :
     Ufunc(1, 1, None,
-          'log(1+x) to base e elementwise.',
+          docstrings.get('numpy.core.umath.log1p'),
           TD(inexact, f='log1p'),
           TD(M, f='log1p'),
           ),
 'sqrt' :
     Ufunc(1, 1, None,
-          'square-root elementwise. For real x, the domain is restricted to x>=0.',
+          docstrings.get('numpy.core.umath.sqrt'),
           TD(inexact, f='sqrt'),
           TD(M, f='sqrt'),
           ),
 'ceil' :
     Ufunc(1, 1, None,
-          'elementwise smallest integer >= x.',
+          docstrings.get('numpy.core.umath.ceil'),
           TD(flts, f='ceil'),
           TD(M, f='ceil'),
           ),
+'trunc' :
+    Ufunc(1, 1, None,
+          docstrings.get('numpy.core.umath.trunc'),
+          TD(flts, f='trunc'),
+          TD(M, f='trunc'),
+          ),
 'fabs' :
     Ufunc(1, 1, None,
-          'absolute values.',
+          docstrings.get('numpy.core.umath.fabs'),
           TD(flts, f='fabs'),
           TD(M, f='fabs'),
        ),
 'floor' :
     Ufunc(1, 1, None,
-          'elementwise largest integer <= x',
+          docstrings.get('numpy.core.umath.floor'),
           TD(flts, f='floor'),
           TD(M, f='floor'),
           ),
 'rint' :
     Ufunc(1, 1, None,
-          'round x elementwise to the nearest integer, round halfway cases away from zero',
+          docstrings.get('numpy.core.umath.rint'),
           TD(inexact, f='rint'),
           TD(M, f='rint'),
           ),
 'arctan2' :
     Ufunc(2, 1, None,
-          'a safe and correct arctan(x1/x2)',
+          docstrings.get('numpy.core.umath.arctan2'),
           TD(flts, f='atan2'),
           TD(M, f='arctan2'),
           ),
 'remainder' :
     Ufunc(2, 1, None,
-          'computes x1-n*x2 where n is floor(x1 / x2)',
+          docstrings.get('numpy.core.umath.remainder'),
           TD(intflt),
           TD(O, f='PyNumber_Remainder'),
           ),
 'hypot' :
     Ufunc(2, 1, None,
-          'sqrt(x1**2 + x2**2) elementwise',
+          docstrings.get('numpy.core.umath.hypot'),
           TD(flts, f='hypot'),
           TD(M, f='hypot'),
           ),
 'isnan' :
     Ufunc(1, 1, None,
-          'returns True where x is Not-A-Number',
+          docstrings.get('numpy.core.umath.isnan'),
           TD(inexact, out='?'),
           ),
 'isinf' :
     Ufunc(1, 1, None,
-          'returns True where x is +inf or -inf',
+          docstrings.get('numpy.core.umath.isinf'),
           TD(inexact, out='?'),
           ),
 'isfinite' :
     Ufunc(1, 1, None,
-          'returns True where x is finite',
+          docstrings.get('numpy.core.umath.isfinite'),
           TD(inexact, out='?'),
           ),
 'signbit' :
     Ufunc(1, 1, None,
-          'returns True where signbit of x is set (x<0).',
+          docstrings.get('numpy.core.umath.signbit'),
           TD(flts, out='?'),
           ),
 'modf' :
     Ufunc(1, 2, None,
-          'breaks x into fractional (y1) and integral (y2) parts.\\n\\n    Each output has the same sign as the input.',
+          docstrings.get('numpy.core.umath.modf'),
           TD(flts),
           ),
 }
@@ -488,27 +596,6 @@ def indent(st,spaces):
     indented = re.sub(r' +$',r'',indented)
     return indented
 
-chartoname = {'?': 'bool',
-              'b': 'byte',
-              'B': 'ubyte',
-              'h': 'short',
-              'H': 'ushort',
-              'i': 'int',
-              'I': 'uint',
-              'l': 'long',
-              'L': 'ulong',
-              'q': 'longlong',
-              'Q': 'ulonglong',
-              'f': 'float',
-              'd': 'double',
-              'g': 'longdouble',
-              'F': 'cfloat',
-              'D': 'cdouble',
-              'G': 'clongdouble',
-              'O': 'OBJECT',
-              'M': 'OBJECT',
-              }
-
 chartotype1 = {'f': 'f_f',
                'd': 'd_d',
                'g': 'g_g',
@@ -531,6 +618,39 @@ chartotype2 = {'f': 'ff_f',
 # 2) fill in functions and data in InitOperators
 # 3) add function.
 
+# String-handling utilities to avoid locale-dependence.
+
+import string
+UPPER_TABLE = string.maketrans(string.ascii_lowercase, string.ascii_uppercase)
+
+def english_upper(s):
+    """ Apply English case rules to convert ASCII strings to all upper case.
+
+    This is an internal utility function to replace calls to str.upper() such
+    that we can avoid changing behavior with changing locales. In particular,
+    Turkish has distinct dotted and dotless variants of the Latin letter "I" in
+    both lowercase and uppercase. Thus, "i".upper() != "I" in a "tr" locale.
+
+    Parameters
+    ----------
+    s : str
+
+    Returns
+    -------
+    uppered : str
+
+    Examples
+    --------
+    >>> from numpy.lib.utils import english_upper
+    >>> english_upper('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_')
+    'ABCDEFGHIJKLMNOPQRSTUVWXYZABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_'
+    >>> english_upper('')
+    ''
+    """
+    uppered = s.translate(UPPER_TABLE)
+    return uppered
+
+
 def make_arrays(funcdict):
     # functions array contains an entry for every type implemented
     #   NULL should be placed where PyUfunc_ style function will be filled in later
@@ -575,11 +695,11 @@ def make_arrays(funcdict):
                 sub += 1
             else:
                 datalist.append('(void *)NULL');
-                tname = chartoname[t.type].upper()
+                tname = english_upper(chartoname[t.type])
                 funclist.append('%s_%s' % (tname, name))
 
             for x in t.in_ + t.out:
-                siglist.append('PyArray_%s' % (chartoname[x].upper(),))
+                siglist.append('PyArray_%s' % (english_upper(chartoname[x]),))
 
             k += 1
 
@@ -601,6 +721,12 @@ def make_ufuncs(funcdict):
     for name in names:
         uf = funcdict[name]
         mlist = []
+        docstring = textwrap.dedent(uf.docstring).strip()
+        docstring = docstring.encode('string-escape').replace(r'"', r'\"')
+        # Split the docstring because some compilers (like MS) do not like big
+        # string literal in C code. We split at endlines because textwrap.wrap
+        # do not play well with \n
+        docstring = '\\n\"\"'.join(docstring.split(r"\n"))
         mlist.append(\
 r"""f = PyUFunc_FromFuncAndData(%s_functions, %s_data, %s_signatures, %d,
                                 %d, %d, %s, "%s",
@@ -608,7 +734,7 @@ r"""f = PyUFunc_FromFuncAndData(%s_functions, %s_data, %s_signatures, %d,
                                                 len(uf.type_descriptions),
                                                 uf.nin, uf.nout,
                                                 uf.identity,
-                                                name, uf.docstring))
+                                                name, docstring))
         mlist.append(r"""PyDict_SetItemString(dictionary, "%s", f);""" % name)
         mlist.append(r"""Py_DECREF(f);""")
         code3list.append('\n'.join(mlist))
