@@ -1,21 +1,30 @@
-""" Build swig, f2py, weave, sources.
+""" Build swig, f2py, pyrex sources.
 """
 
 import os
 import re
 import sys
+import shlex
 
 from distutils.command import build_ext
 from distutils.dep_util import newer_group, newer
 from distutils.util import get_platform
 from distutils.errors import DistutilsError, DistutilsSetupError
 
+try:
+    import Pyrex.Compiler.Main
+    have_pyrex = True
+except ImportError:
+    have_pyrex = False
+
+# this import can't be done here, as it uses numpy stuff only available
+# after it's installed
+#import numpy.f2py
 from numpy.distutils import log
 from numpy.distutils.misc_util import fortran_ext_match, \
      appendpath, is_string, is_sequence
 from numpy.distutils.from_template import process_file as process_f_file
 from numpy.distutils.conv_template import process_file as process_c_file
-from numpy.distutils.exec_command import splitcmdline
 
 class build_src(build_ext.build_ext):
 
@@ -56,7 +65,6 @@ class build_src(build_ext.build_ext):
         self.swig_opts = None
         self.swig_cpp = None
         self.swig = None
-        return
 
     def finalize_options(self):
         self.set_undefined_options('build',
@@ -86,7 +94,7 @@ class build_src(build_ext.build_ext):
         if self.f2py_opts is None:
             self.f2py_opts = []
         else:
-            self.f2py_opts = splitcmdline(self.f2py_opts)
+            self.f2py_opts = shlex.split(self.f2py_opts)
 
         if self.swigflags:
             if self.swig_opts:
@@ -95,10 +103,10 @@ class build_src(build_ext.build_ext):
                 self.swig_opts = self.swigflags
             self.swigflags = None
 
-        if self.swig_opts is None: 
+        if self.swig_opts is None:
             self.swig_opts = []
         else:
-            self.swig_opts = splitcmdline(self.swig_opts)
+            self.swig_opts = shlex.split(self.swig_opts)
 
         # use options from build_ext command
         build_ext = self.get_finalized_command('build_ext')
@@ -108,27 +116,24 @@ class build_src(build_ext.build_ext):
             self.swig_cpp = build_ext.swig_cpp
         for c in ['swig','swig_opt']:
             o = '--'+c.replace('_','-')
-            v = getattr(build_ext,c,None) 
+            v = getattr(build_ext,c,None)
             if v:
                 if getattr(self,c):
                     log.warn('both build_src and build_ext define %s option' % (o))
                 else:
                     log.info('using "%s=%s" option from build_ext command' % (o,v))
                     setattr(self, c, v)
-        return
 
     def run(self):
         if not (self.extensions or self.libraries):
             return
         self.build_sources()
 
-        return
-
     def build_sources(self):
 
         if self.inplace:
-            self.get_package_dir = self.get_finalized_command('build_py')\
-                                   .get_package_dir
+            self.get_package_dir = \
+                     self.get_finalized_command('build_py').get_package_dir
 
         self.build_py_modules_sources()
 
@@ -143,8 +148,6 @@ class build_src(build_ext.build_ext):
 
         self.build_data_files_sources()
 
-        return
-
     def build_data_files_sources(self):
         if not self.data_files:
             return
@@ -179,7 +182,6 @@ class build_src(build_ext.build_ext):
             else:
                 raise TypeError(repr(data))
         self.data_files[:] = new_data_files
-        return
 
     def build_py_modules_sources(self):
         if not self.py_modules:
@@ -200,13 +202,12 @@ class build_src(build_ext.build_ext):
                 if source is None:
                     continue
                 modules = [(package, module_base, source)]
-                if not self.py_modules_dict.has_key(package):
+                if package not in self.py_modules_dict:
                     self.py_modules_dict[package] = []
                 self.py_modules_dict[package] += modules
             else:
                 new_py_modules.append(source)
         self.py_modules[:] = new_py_modules
-        return
 
     def build_library_sources(self, lib_name, build_info):
         sources = list(build_info.get('sources',[]))
@@ -223,7 +224,8 @@ class build_src(build_ext.build_ext):
         sources, h_files = self.filter_h_files(sources)
 
         if h_files:
-            print self.package,'- nothing done with h_files=',h_files
+            log.info('%s - nothing done with h_files = %s',
+                     self.package, h_files)
 
         #for f in h_files:
         #    self.distribution.headers.append((lib_name,f))
@@ -257,7 +259,7 @@ class build_src(build_ext.build_ext):
 
         sources, py_files = self.filter_py_files(sources)
 
-        if not self.py_modules_dict.has_key(package):
+        if package not in self.py_modules_dict:
             self.py_modules_dict[package] = []
         modules = []
         for f in py_files:
@@ -268,14 +270,13 @@ class build_src(build_ext.build_ext):
         sources, h_files = self.filter_h_files(sources)
 
         if h_files:
-            print package,'- nothing done with h_files=',h_files
+            log.info('%s - nothing done with h_files = %s',
+                     package, h_files)
         #for f in h_files:
         #    self.distribution.headers.append((package,f))
 
         ext.sources = sources
 
-        return
-
     def generate_sources(self, sources, extension):
         new_sources = []
         func_sources = []
@@ -291,7 +292,7 @@ class build_src(build_ext.build_ext):
         else:
             if is_sequence(extension):
                 name = extension[0]
-            #    if not extension[1].has_key('include_dirs'):
+            #    if 'include_dirs' not in extension[1]:
             #        extension[1]['include_dirs'] = []
             #    incl_dirs = extension[1]['include_dirs']
             else:
@@ -370,46 +371,48 @@ class build_src(build_ext.build_ext):
         return new_sources
 
     def pyrex_sources(self, sources, extension):
-        have_pyrex = False
-        try:
-            import Pyrex
-            have_pyrex = True
-        except ImportError:
-            pass
         new_sources = []
         ext_name = extension.name.split('.')[-1]
         for source in sources:
             (base, ext) = os.path.splitext(source)
             if ext == '.pyx':
-                if self.inplace or not have_pyrex:
-                    target_dir = os.path.dirname(base)
-                else:
-                    target_dir = appendpath(self.build_src, os.path.dirname(base))
-                target_file = os.path.join(target_dir, ext_name + '.c')
-                depends = [source] + extension.depends
-                if (self.force or newer_group(depends, target_file, 'newer')):
-                    if have_pyrex:
-                        log.info("pyrexc:> %s" % (target_file))
-                        self.mkpath(target_dir)
-                        from Pyrex.Compiler import Main
-                        options = Main.CompilationOptions(
-                            defaults=Main.default_options,
-                            output_file=target_file)
-                        pyrex_result = Main.compile(source, options=options)
-                        if pyrex_result.num_errors != 0:
-                            raise DistutilsError,"%d errors while compiling %r with Pyrex" \
-                                  % (pyrex_result.num_errors, source)
-                    elif os.path.isfile(target_file):
-                        log.warn("Pyrex required for compiling %r but not available,"\
-                                 " using old target %r"\
-                                 % (source, target_file))
-                    else:
-                        raise DistutilsError,"Pyrex required for compiling %r but not available" % (source)
+                target_file = self.generate_a_pyrex_source(base, ext_name,
+                                                           source,
+                                                           extension)
                 new_sources.append(target_file)
             else:
                 new_sources.append(source)
         return new_sources
 
+    def generate_a_pyrex_source(self, base, ext_name, source, extension):
+        if self.inplace or not have_pyrex:
+            target_dir = os.path.dirname(base)
+        else:
+            target_dir = appendpath(self.build_src, os.path.dirname(base))
+        target_file = os.path.join(target_dir, ext_name + '.c')
+        depends = [source] + extension.depends
+        if self.force or newer_group(depends, target_file, 'newer'):
+            if have_pyrex:
+                log.info("pyrexc:> %s" % (target_file))
+                self.mkpath(target_dir)
+                options = Pyrex.Compiler.Main.CompilationOptions(
+                    defaults=Pyrex.Compiler.Main.default_options,
+                    include_path=extension.include_dirs,
+                    output_file=target_file)
+                pyrex_result = Pyrex.Compiler.Main.compile(source,
+                                                           options=options)
+                if pyrex_result.num_errors != 0:
+                    raise DistutilsError,"%d errors while compiling %r with Pyrex" \
+                          % (pyrex_result.num_errors, source)
+            elif os.path.isfile(target_file):
+                log.warn("Pyrex required for compiling %r but not available,"\
+                         " using old target %r"\
+                         % (source, target_file))
+            else:
+                raise DistutilsError("Pyrex required for compiling %r"\
+                                     " but notavailable" % (source,))
+        return target_file
+
     def f2py_sources(self, sources, extension):
         new_sources = []
         f2py_sources = []
@@ -462,7 +465,8 @@ class build_src(build_ext.build_ext):
         if not (f2py_sources or f_sources):
             return new_sources
 
-        map(self.mkpath, target_dirs)
+        for d in target_dirs:
+            self.mkpath(d)
 
         f2py_options = extension.f2py_options + self.f2py_opts
 
@@ -485,8 +489,9 @@ class build_src(build_ext.build_ext):
             if (self.force or newer_group(depends, target_file,'newer')) \
                    and not skip_f2py:
                 log.info("f2py: %s" % (source))
-                import numpy.f2py as f2py2e
-                f2py2e.run_main(f2py_options + ['--build-dir',target_dir,source])
+                import numpy.f2py
+                numpy.f2py.run_main(f2py_options
+                                    + ['--build-dir',target_dir,source])
             else:
                 log.debug("  skipping '%s' f2py interface (up-to-date)" % (source))
         else:
@@ -501,10 +506,10 @@ class build_src(build_ext.build_ext):
             depends = f_sources + extension.depends
             if (self.force or newer_group(depends, target_file, 'newer')) \
                    and not skip_f2py:
-                import numpy.f2py as f2py2e
                 log.info("f2py:> %s" % (target_file))
                 self.mkpath(target_dir)
-                f2py2e.run_main(f2py_options + ['--lower',
+                import numpy.f2py
+                numpy.f2py.run_main(f2py_options + ['--lower',
                                                 '--build-dir',target_dir]+\
                                 ['-m',ext_name]+f_sources)
             else:
@@ -524,8 +529,8 @@ class build_src(build_ext.build_ext):
             extension.include_dirs.append(self.build_src)
 
         if not skip_f2py:
-            import numpy.f2py as f2py2e
-            d = os.path.dirname(f2py2e.__file__)
+            import numpy.f2py
+            d = os.path.dirname(numpy.f2py.__file__)
             source_c = os.path.join(d,'src','fortranobject.c')
             source_h = os.path.join(d,'src','fortranobject.h')
             if newer(source_c,target_c) or newer(source_h,target_h):
@@ -628,7 +633,9 @@ class build_src(build_ext.build_ext):
         if skip_swig:
             return new_sources + py_files
 
-        map(self.mkpath, target_dirs)
+        for d in target_dirs:
+            self.mkpath(d)
+
         swig = self.swig or self.find_swig()
         swig_cmd = [swig, "-python"]
         if is_cpp:
