@@ -1,12 +1,9 @@
 import os
 import sys
-import pkgutil
 import types
 import re
 
-from numpy.core.numerictypes import obj2sctype, generic, issubclass_, \
-    issubsctype, issubdtype
-from numpy.core.multiarray import dtype as _dtype
+from numpy.core.numerictypes import issubclass_, issubsctype, issubdtype
 from numpy.core import product, ndarray
 
 __all__ = ['issubclass_', 'get_numpy_include', 'issubsctype',
@@ -45,15 +42,23 @@ def get_include():
     return d
 
 def get_numarray_include(type=None):
-    """Return the directory in the package that contains the numpy/*.h header
-    files.
+    """
+    Return the directory that contains the numarray \\*.h header files.
 
-    Extension modules that need to compile against numpy should use this
-    function to locate the appropriate include directory. Using distutils:
+    Extension modules that need to compile against numarray should use this
+    function to locate the appropriate include directory.
+
+    Notes
+    -----
+    When using ``distutils``, for example in ``setup.py``.
+    ::
+
+        import numpy as np
+        ...
+        Extension('extension_name', ...
+                include_dirs=[np.get_numarray_include()])
+        ...
 
-      import numpy
-      Extension('extension_name', ...
-                include_dirs=[numpy.get_numarray_include()])
     """
     from numpy.numarray import get_numarray_include_dirs
     include_dirs = get_numarray_include_dirs()
@@ -99,10 +104,7 @@ def deprecate(func, oldname=None, newname=None):
         depdoc = '%s is DEPRECATED!! -- use %s instead' % (oldname, newname,)
 
     def newfunc(*args,**kwds):
-        """
-        Use get_include, get_numpy_include is DEPRECATED.
-
-        """
+        """Use get_include, get_numpy_include is DEPRECATED."""
         warnings.warn(str1, DeprecationWarning)
         return func(*args, **kwds)
 
@@ -147,15 +149,21 @@ get_numpy_include = deprecate(get_include, 'get_numpy_include', 'get_include')
 #--------------------------------------------
 
 def byte_bounds(a):
-    """(low, high) are pointers to the end-points of an array
+    """
+    Returns pointers to the end-points of an array.
 
-    low is the first byte
-    high is just *past* the last byte
+    Parameters
+    ----------
+    a : ndarray
+        Input array. It must conform to the Python-side of the array interface.
 
-    If the array is not single-segment, then it may not actually
-    use every byte between these bounds.
+    Returns
+    -------
+    (low, high) : tuple of 2 integers
+        The first integer is the first byte of the array, the second integer is
+        just past the last byte of the array.  If `a` is not contiguous it
+        would not use every byte between the (`low`, `high`) values.
 
-    The array provided must conform to the Python-side of the array interface
     """
     ai = a.__array_interface__
     a_data = ai['data'][0]
@@ -226,10 +234,8 @@ def who(vardict=None):
     >>> np.whos(d)
     Name            Shape            Bytes            Type
     ===========================================================
-    <BLANKLINE>
     y               3                24               float64
     x               2                16               float64
-    <BLANKLINE>
     Upper bound on total bytes  =       40
 
     """
@@ -338,18 +344,30 @@ def _makenamedict(module='numpy'):
     return thedict, dictlist
 
 def info(object=None,maxwidth=76,output=sys.stdout,toplevel='numpy'):
-    """Get help information for a function, class, or module.
+    """
+    Get help information for a function, class, or module.
 
-       Example:
-          >>> np.info(np.polyval) # doctest: +SKIP
+    Parameters
+    ----------
+    object : optional
+        Input object to get information about.
+    maxwidth : int, optional
+        Printing width.
+    output : file like object open for writing, optional
+        Write into file like object.
+    toplevel : string, optional
+        Start search at this level.
 
-          polyval(p, x)
+    Examples
+    --------
+    >>> np.info(np.polyval) # doctest: +SKIP
 
-            Evaluate the polymnomial p at x.
+       polyval(p, x)
+
+         Evaluate the polymnomial p at x.
+
+         ...
 
-            Description:
-                If p is of length N, this function returns the value:
-                p[0]*(x**N-1) + p[1]*(x**N-2) + ... + p[N-2]*x + p[N-1]
     """
     global _namedict, _dictlist
     # Local import to speed up numpy's import time.
@@ -682,15 +700,21 @@ def _lookfor_generate_cache(module, import_modules, regenerate):
                 _all = item.__all__
             except AttributeError:
                 _all = None
+
             # import sub-packages
             if import_modules and hasattr(item, '__path__'):
-                for m in pkgutil.iter_modules(item.__path__):
-                    if _all is not None and m[1] not in _all:
-                        continue
-                    try:
-                        __import__("%s.%s" % (name, m[1]))
-                    except ImportError:
-                        continue
+                for pth in item.__path__:
+                    for mod_path in os.listdir(pth):
+                        init_py = os.path.join(pth, mod_path, '__init__.py')
+                        if not os.path.isfile(init_py):
+                            continue
+                        if _all is not None and mod_path not in _all:
+                            continue
+                        try:
+                            __import__("%s.%s" % (name, mod_path))
+                        except ImportError:
+                            continue
+
             for n, v in inspect.getmembers(item):
                 if _all is not None and n not in _all:
                     continue
